@@ -11,7 +11,6 @@ import inspect
 import logging
 import time
 import json
-import random
 
 from openrazer_daemon.dbus_services.service import DBusService
 import openrazer_daemon.dbus_services.dbus_methods
@@ -47,7 +46,7 @@ class RazerDevice(DBusService):
 
     DEVICE_IMAGE = None
 
-    def __init__(self, device_path, device_number, config, persistence, testing, additional_interfaces, additional_methods):
+    def __init__(self, device_path, device_number, config, persistence, testing, additional_interfaces, additional_methods, unknown_serial_counter):
 
         self.logger = logging.getLogger('razer.device{0}'.format(device_number))
         self.logger.info("Initialising device.%d %s", device_number, self.__class__.__name__)
@@ -55,6 +54,9 @@ class RazerDevice(DBusService):
         # Serial cache
         self._serial = None
 
+        # map of vid+pid to counter for serial numbers for unknown devices
+        self._unknown_serial_counter: dict[tuple[int, int], int] = unknown_serial_counter
+
         # Local storage key name
         self.storage_name = "UnknownDevice"
 
@@ -326,6 +328,11 @@ class RazerDevice(DBusService):
         if self.config.getboolean('Startup', "restore_persistence") is True:
             self.restore_effect()
 
+            # Some devices need setting a second time after encountering Razer Synapse on Windows
+            if self.config.getboolean('Startup', "persistence_dual_boot_quirk") is True:
+                self.logger.debug("Restoring effect persistence again (dual boot quirk)")
+                self.restore_effect()
+
     def send_effect_event(self, effect_name, *args):
         """
         Send effect event
@@ -974,8 +981,19 @@ class RazerDevice(DBusService):
                     time.sleep(0.1)
                     self.logger.debug('getting serial: {0} count:{1}'.format(serial, count))
 
-            if serial == '' or serial == 'Default string' or serial == 'empty (NULL)' or serial == 'As printed in the D cover':
-                serial = 'UNKWN{0:012}'.format(random.randint(0, 4096))
+            # Known bad serials:
+            # - just an empty string
+            # - "Default string"
+            # - "empty (NULL)"
+            # - "As printed in the D cover"
+            # - hex: 01 01 01 01 05 06 07 08 09 0a 0b 0c 0d 0e 0f 10 11 12 13 14 15 16
+            if not re.fullmatch(r"[A-Z]+[\dA-Z]+", serial):
+                self.logger.warning("Invalid serial number found, using a generated one.")
+                self.logger.warning("Original value: %s" % serial)
+                vid, pid = self.get_vid_pid()
+                idx = self._unknown_serial_counter.get((vid, pid), 0)
+                self._unknown_serial_counter[(vid, pid)] = idx + 1
+                serial = "UNKNOWN_{0:04X}{1:04X}_{2:04d}".format(vid, pid, idx)
 
             self._serial = serial.replace(' ', '_')
 
