@@ -1,13 +1,15 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
- * Copyright (c) 2015 Terry Cain <terry@terrys-home.co.uk>
+ * Copyright (c) 2015 Terri Cain <terri@dolphincorp.co.uk>
  */
 
 #ifndef __HID_RAZER_MOUSE_H
 #define __HID_RAZER_MOUSE_H
 
 #define USB_DEVICE_ID_RAZER_OROCHI_2011 0x0013
+#define USB_DEVICE_ID_RAZER_NAGA 0x0015
 #define USB_DEVICE_ID_RAZER_DEATHADDER_3_5G 0x0016
+#define USB_DEVICE_ID_RAZER_NAGA_EPIC 0x001F
 #define USB_DEVICE_ID_RAZER_ABYSSUS_1800 0x0020
 #define USB_DEVICE_ID_RAZER_MAMBA_2012_WIRED 0x0024
 #define USB_DEVICE_ID_RAZER_MAMBA_2012_WIRELESS 0x0025
@@ -96,6 +98,15 @@
 #define USB_DEVICE_ID_RAZER_DEATHADDER_V3_PRO_WIRELESS 0x00B7
 #define USB_DEVICE_ID_RAZER_VIPER_V3_HYPERSPEED 0x00B8
 #define USB_DEVICE_ID_RAZER_BASILISK_V3_X_HYPERSPEED 0x00B9
+#define USB_DEVICE_ID_RAZER_VIPER_V3_PRO_WIRED 0x00C0
+#define USB_DEVICE_ID_RAZER_VIPER_V3_PRO_WIRELESS 0x00C1
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V3_PRO_WIRED_ALT 0x00C2
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V3_PRO_WIRELESS_ALT 0x00C3
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V3_HYPERSPEED_WIRED 0x00C4
+#define USB_DEVICE_ID_RAZER_DEATHADDER_V3_HYPERSPEED_WIRELESS 0x00C5
+#define USB_DEVICE_ID_RAZER_BASILISK_V3_35K 0x00CB
+#define USB_DEVICE_ID_RAZER_BASILISK_V3_PRO_35K_WIRED 0x00CC
+#define USB_DEVICE_ID_RAZER_BASILISK_V3_PRO_35K_WIRELESS 0x00CD
 
 /* Each keyboard report has 90 bytes*/
 #define RAZER_REPORT_LEN 0x5A
